// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _GLOBALS_H_
#define _GLOBALS_H_

#include <string>

#define HOBBES_VERSION "2.0"
#define HOBBES_PROGRAM_NAME "Hobbes2"
#define SAM_VERSION "1.4"

#ifndef BITVECTOR_SIZE
// Default bitvector size is 16 bits (2 bytes)
#define BITVECTOR_SIZE 2
#endif

#if BITVECTOR_SIZE == 1
typedef unsigned char BitvectorType;
#define GCC_POPCOUNT __builtin_popcount
#endif

#if BITVECTOR_SIZE == 2
typedef unsigned short BitvectorType;
#define GCC_POPCOUNT __builtin_popcount
#endif

#if BITVECTOR_SIZE == 4
typedef unsigned int BitvectorType;
#define GCC_POPCOUNT __builtin_popcount
#endif

#if BITVECTOR_SIZE == 8
typedef unsigned long BitvectorType;
#define GCC_POPCOUNT __builtin_popcountl
#endif


#ifndef PPFILTER_CHUNKSIZE
#define PPFILTER_CHUNKSIZE 4
#endif

#if PPFILTER_CHUNKSIZE == 2
typedef unsigned short PPFilterChunkType;
#endif

#if PPFILTER_CHUNKSIZE == 4
typedef unsigned int PPFilterChunkType;
#endif

#if PPFILTER_CHUNKSIZE == 8
typedef unsigned long PPFilterChunkType;
#endif

// Alphabets.
const char BASE_A = 'A';
const char BASE_G = 'G';
const char BASE_T = 'T';
const char BASE_C = 'C';
const char BASE_N = 'N';

const char BASE_A_SMALL = 'a';
const char BASE_G_SMALL = 'g';
const char BASE_T_SMALL = 't';
const char BASE_C_SMALL = 'c';
const char BASE_N_SMALL = 'n';

// For output (Native, SAM).
const char FWD_MAPPING = '+';
const char REV_MAPPING = '-';

// SAM tags

// Header tags.
const std::string HD_TAG = "@HD";
const std::string VERSION_TAG = "VN:";
const std::string SO_TAG = "SO:";
const std::string SEQ_TAG = "@SQ";
const std::string SEQNAME_TAG = "SN:";
const std::string SEQLEN_TAG = "LN:";
const std::string PROG_TAG = "@PG	ID:Hobbes";
const std::string PROGNAME_TAG = "PN:Hobbes2";
const std::string CMD_LINE_TAG = "CL:";

// Result tags.
const char CIGAR = 'M';
const std::string N_TAG = "N:";
const std::string NM_TAG = "NM:i:";
const std::string XM_TAG = "XM:i:";
const std::string XA_Z_TAG = "XA:Z:";
const std::string XA_I_TAG = "XA:i:";
const std::string MD_Z_TAG = "MD:Z:";
const char UNAVAILABLE_1 = '*';
const char UNAVAILABLE_2 = '0';
const short unsigned UNAVAILABLE_3 = 255;

// Formatting.
const std::string SEPARATOR = "\t";
const char COMMA = ',';
const char SEMI_COLON = ';';
#endif
