// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _HOBBESUSERHELP_H_
#define _HOBBESUSERHELP_H_

#define S(x) #x

const char* user_help =
S(Usage:\n)
S(\thobbes-index [options] - Creates a Hobbes index.\n)
S(\thobbes [options] - Performs read-mapping using an existing Hobbes index.\n)
S(\n)
S(hobbes-index options:\n)
S(-g <int> = Use given gram length to build a Hobbes index. We recommend a gram length of 11. We support gram lengths up to 16 but the index size will increase dramatically after gram length 13.\n)
S(-i <file> = Create Hobbes index into given file.\n)
S(--sref <file> = Reference sequence file to index in fasta format.\n)
S(--dref <file> = Uses all fasta files in given directory as reference sequence. File names become chromosome names.\n)
S(-p <int> = Use <int> parallel pthreads to construct the index.\n)
S(\n)
S(hobbes options:\n)
S(Read input options:\n)
S(-q <file> = Map single-end reads in given fastq file.\n)
S(-r <file> = Map single-end reads in given line-by-line text file.\n)
S(-f <file> = Map single-end reads in given fasta file.\n)
S(-c <string> = Map given single-end read (only maps a single read).\n)
S(--seqfq1 = First FastQ file for paired-end reads. Requires --pe.\n)
S(--seqfq2 = Second FastQ file for paired-end reads. Requires --pe.\n)
S(--gzip = Reads file is compressed with gzip.\n)
S(--bzip2 = Reads file is compressed with bzip2.\n)
S(\n)
S(Reference sequence input options:\n)
S(--sref <file> = Reference sequence file in fasta format.\n)
S(--dref <file> = Uses all fasta files in given directory as reference sequence. File names become chromosome names.\n)
S(\n)
S(Hobbes index input options:\n)
S(-i <file> = Use given Hobbes index to perform mapping.\n)
S(\n)
S(Paired-end mapping:\n)
S(--pe = Enable paired-end mapping mode. See --seqfq1 and --seqfq2.\n)
S(--min = Minimum insert size.\n)
S(--max = Maximum insert size.\n)
S(\n)
S(Mapping modes:\n)
S(--hamming = Map reads using using Hamming distance. This is the fastest mode of Hobbes.\n)
S(--indel = Map reads using edit distance. Uses heuristics to speed up the search and is not guaranteed to find the best possible mapping locations (but very often it does).\n)
S(-v <int> = Distance threshold. Finds reads within given distance threshold (use --hamming for Hammign distance and --indel for edit distance).\n)
S(-a = Find all mapping locations.\n)
S(-u = Find unique mappings only.\n)
S(-k <int> = Find upto 'k' mappings per read.\n)
S(-n <int> = Aligns first <int> reads. By default all the reads are aligned. Required for useful progress indication.\n)
S(--norc = Aligns against forward reference only.\n)
S(--nofw = Aligns against reverse reference only.\n)
S(The default option aligns against forward and reverse reference.\n)
S(-p <int> = Runs <int> parallel pthreads to perform the mapping.\n)
S(--nocigar = Map reads without calculating cigar strings. Usually faster than the regular mode.\n)
S(\n)
S(Output options:\n)
S(Default = Uses SAM output format with cigar string.\n)
S(--mapout <file> = Prints the mapping results to file <file>. By default the results are written to standard ouput.\n)
S(\n)
S(SAM:\n)
S(--sam-nohead = Suppresses the header lines (starting with '@'). This must be used with '-S'.\n)
S(--sam-nosq = Suppresses the @SQ header lines. This must be used with '-S'.\n)
S(\n)
S(Other options:\n)
S(--noprogress = Disable progress indicator.\n)
S(--version = Prints the VERSION file.\n)
S(--help = Prints this file.);

#endif
