// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _INDEXBUILDER_H_
#define _INDEXBUILDER_H_

#include "thread/indexbuildworker.h"

#include <vector>
#include <string>

class IndexPartition {
public:
	unsigned startPos;
	unsigned endPos;

	IndexPartition(unsigned startPos, unsigned endPos) :
		startPos(startPos), endPos(endPos) {
	}

	// For sorting descending by size of partition.
	bool operator<(const IndexPartition& ixp) const {
		return (endPos - startPos) >= (ixp.endPos - ixp.startPos);
	}
};

template<class T>
class HeapElement {
public:
	typedef T* iterator;
	T* iter;
	T* end;

	HeapElement(T* iter, T* end) :
		iter(iter), end(end) {
	}

	bool operator<(const HeapElement& he) const {
		if ((iter->hash) == (he.iter->hash)) {
			return (iter->bitVecPair.pos) >= (he.iter->bitVecPair.pos);
		}
		return (iter->hash) >= (he.iter->hash);
	}
};

template<class ElementType>
class IndexBuilder {
private:
	const std::string& refSeq;
	const char* indexFileName;
	unsigned gramLength;
	unsigned numThreads;

	std::vector<IndexPartition> indexPartitions;

	ElementType* allElements;

	void mergeToFile(std::vector<IndexBuildWorker<BitVecTriple>* >& workerPtrs, bool useProgressIndicator);

public:
	IndexBuilder(const std::string& refSeq, const char* indexFileName, unsigned gramLength,
			unsigned numThreads) :
		refSeq(refSeq), indexFileName(indexFileName), gramLength(gramLength),
		numThreads(numThreads), allElements(NULL) {
	}

	void addIndexPartition(IndexPartition ixp) {
		indexPartitions.push_back(ixp);
	}

	void build(bool useProgressIndicator);
};


#endif
