// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _INPUTREADQUEUE_H_
#define _INPUTREADQUEUE_H_

#include <string>
#include <queue>
#include <pthread.h>

#include <input/readinfo.h>

enum GetReadState {
	GRS_ALL_DONE, GRS_OK
};

const std::string READ_QUAL_UNAVAIL = "*";

class InputReadQueue {
protected:
	std::queue<ReadInfo*> readQueue;
	pthread_mutex_t queueMutex;
	pthread_cond_t consumerCond;
	pthread_cond_t producerCond;
	unsigned maxReads;
	unsigned minOutstanding;
	unsigned maxOutstanding;
	unsigned readsEnqueued;
	unsigned readsDequeued;
	bool nextReadFound;

	// To be implemented by sub-classes.
	virtual bool nextRead() = 0;

public:
	InputReadQueue(unsigned maxReads, unsigned minOutstanding, unsigned maxOutstanding);
	virtual ~InputReadQueue();

	// Retrieve the length of the first read.
        // Must be called before InputReadQueue is executed as a thread
	int estimateReadLength();

	// Start pushing reads to the queue, meant to be executed as a thread.
	void enqueueReads();

	// Get a read from the queue, calls nextRead() internally.
	// Returns true if more reads available, else false.
	bool getRead(ReadInfo** inputReadStruct);

	unsigned getReadsEnqueued() const {
		return readsEnqueued;
	}
	unsigned getReadsDequeued() const {
		return readsDequeued;
	}
};

#endif
