// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#include "globals.h"
#include "hobbesrunner.h"
#include "hobbesconf.h"
#include "hobbesstats.h"
#include "hobbesuserhelp.h"

#include <iostream>
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <getopt.h>

using namespace std;

const char* optString = "tp:g:i:";
static const struct option longOpts[] = {
    { "show time", no_argument, NULL, 't' },
    { "pthreads", required_argument, NULL, 'p' },
    { "gram length", required_argument, NULL, 'g' },
    { "index file", required_argument, NULL, 'i' },
    { "sref", required_argument, NULL, 0 },	//ref seq file
    { "dref", required_argument, NULL, 0 },	//ref seq dir
    { "version", no_argument, NULL, 0 },
    { "help", no_argument, NULL, 0 },
    { NULL, no_argument, NULL, 0 }
};

int main (int argc, char **argv) {
	// Print help if no args given.
	if (argc == 1) {
		clog << user_help << endl;
		exit(0);
	}

	HobbesRunner runner;

	vector<unsigned> complexOpts;

	int longIndex;
	int opt = getopt_long(argc, argv, optString, longOpts, &longIndex);
	while (opt != -1) {
		switch (opt) {
		case 't':
			runner.conf.useProgressIndicator = true;
			break;
		case 'i' :
			runner.conf.indexFileName = optarg;
			break;
		case 'p':
			runner.conf.numThreads = atoi(optarg);
			break;
		case 'g':
			runner.conf.gramLength = atoi(optarg);
			break;
		case 0:
			if(strcmp("sref", longOpts[longIndex].name) == 0){
				runner.conf.refSeqPathIsDir = false;
				runner.conf.refSeqPath = optarg;
			} else	if(strcmp("dref", longOpts[longIndex].name) == 0) {
				runner.conf.refSeqPathIsDir = true;
				runner.conf.refSeqPath = optarg;
			} else if (strcmp("version", longOpts[longIndex].name) == 0) {
				clog << HOBBES_VERSION << " [" << HOBBES_PROGRAM_NAME << "]" << endl;
				exit(0);
			} else if (strcmp("help", longOpts[longIndex].name) == 0) {
				clog << user_help << endl;
				exit(0);
			}
		break;
		default: {
			abort ();
		} break;

		}
		opt = getopt_long(argc, argv, optString, longOpts, &longIndex);
	}

	runner.conf.indexOptions = CREATE_INDEX;
	runner.conf.outputOptions |= WRITE_STATS;
	runner.conf.checkIndexConstructionConf();
	runner.run();
	return 0;
}
