// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#include "thread/searchworker.h"
#include "search/searcher.h"
#include "input/inputreadqueue.h"
#include "output/resultstringwriter.h"
#include "output/outputresultsink.h"
#include "tools/progressindicator.h"

#include <cstdlib>

using namespace std;

#ifndef SELECTION_SEARCH_WORKER_PROGRESS_MIN
#define SELECTION_SEARCH_WORKER_PROGRESS_MIN 100
#endif

#ifndef SELECTION_SEARCH_WORKER_PROGRESS_MAX
#define SELECTION_SEARCH_WORKER_PROGRESS_MAX 1000
#endif

using namespace std;
void SearchWorker::run() {
	srand(time(NULL));
	unsigned updateInterval = rand() % (SELECTION_SEARCH_WORKER_PROGRESS_MAX - SELECTION_SEARCH_WORKER_PROGRESS_MIN) + SELECTION_SEARCH_WORKER_PROGRESS_MIN;
	ReadInfo* readInfo = NULL;
	MappingResult result;
	readCount = 0;
	while(inrq->getRead(&readInfo)) {
		readCount++;
		stringstream* ss = new stringstream();
		searcher->findMappings(*readInfo);
		result.stream = ss;
		result.type = searcher->getMappingResultType();
		result.readNum = readInfo->readNum;
		resultWriter->writeResultString(*ss, *readInfo, searcher);
		outSink->add(result);
		delete readInfo;
		readInfo = NULL;
		if(progressIndicator && (readCount % updateInterval == 0)) {
			progressIndicator->incrementProgress(updateInterval);
		}
	}
}
